#include <iostream>
#include <fstream>
#include <iomanip>
#include <string> 
#include <math.h>
#include <stdexcept>

using namespace std;

using funkce = double(*)(double);

struct popis_funkce
{
	funkce f;
	string jmeno;
};

struct popis_tabulky
{
	double od, po;
	int kolik;
};

struct popis_vystupu
{
	int sirka_x, sirka_f, presnost_x, presnost_f;
	ostream& proud;
};

void kontrola(popis_funkce fun, popis_tabulky tab)
{
	if (!fun.f)
	{
		throw invalid_argument("Przdn ukazatel na tabelovanou funkci");
	}
	if (tab.kolik <= 0) 
	{
		throw invalid_argument("Nekladn poet poadovanch hodnot");
	}
	if ((tab.kolik == 1) && (tab.od != tab.po))
	{
		throw invalid_argument("Poadovna jedin hodnota v intervalu nenulov dlky");
	}
}

void zahlavi(string jmeno, int sirka_x, int sirka_f, ostream& proud)
{
	int pocet_pomlcek = sirka_x + 10 + sirka_f;
	proud << "Tabulka funkce " << jmeno << endl
		<< string(pocet_pomlcek, '-') << endl << setw(sirka_x)
		<< "x" << string(5 + sirka_x, ' ') << setw(sirka_f) << jmeno << endl
		<< string(pocet_pomlcek, '-') << endl;
}

void tabulka(popis_funkce fun, popis_tabulky tab, popis_vystupu vyst)
{
	kontrola(fun, tab);
	double krok = tab.kolik == 1 ? 0 : (tab.po - tab.od) / (tab.kolik - 1);
	zahlavi(fun.jmeno, vyst.sirka_x, vyst.sirka_f, vyst.proud);
	vyst.proud << fixed;
	for (int i = 0; i < tab.kolik; i++)
	{
		double x = tab.od + i*krok;
		vyst.proud << setprecision(vyst.presnost_x) << setw(vyst.sirka_x) << x << "         "
			<< setprecision(vyst.presnost_f) << setw(vyst.sirka_f) << fun.f(x) << endl;
	}
}

int main()
{
	try
	{
		ofstream proud("tabulka.txt");
		popis_funkce f1{ [](double x)->double {return x*x; }, "x^2"};
		popis_tabulky tab1{ 998,1008,11 };
		popis_vystupu vyst1{ 4,8,0,0,proud };
		tabulka(f1, tab1, vyst1);
		proud << "**************" << endl;
		popis_funkce f2{ sin, "sin(x)" };
		popis_tabulky tab2{ 0,1,11 };
		popis_vystupu vyst2{ 4,7,2,5, proud };
		tabulka(f2, tab2, vyst2);
	}
	catch (invalid_argument e)
	{
		cerr << e.what() << endl;
	}
}

